@@ -8,7 +8,8 @@ import translationstring
 
 _ = translationstring.TranslationStringFactory('colander')
 
-_marker = object()
+required = object()
+_marker = required # bw compat
 
 class _null(object):
     """ Represents a null value in colander-related operations. """
@@ -452,6 +453,8 @@ class Mapping(SchemaType):
         return self._impl(node, appstruct, callback)
 
     def deserialize(self, node, cstruct):
+        if cstruct is null:
+            return null
 
         def callback(subnode, subcstruct):
             return subnode.deserialize(subcstruct)
@@ -542,6 +545,9 @@ class Tuple(Positional, SchemaType):
         return self._impl(node, appstruct, callback)
 
     def deserialize(self, node, cstruct):
+        if cstruct is null:
+            return null
+        
         def callback(subnode, subval):
             return subnode.deserialize(subval)
 
@@ -670,6 +676,9 @@ class Sequence(Positional, SchemaType):
         respect the default ``accept_scalar`` value attached to this
         instance via its constructor.
         """
+        if cstruct is null:
+            return null
+        
         def callback(subnode, subcstruct):
             return subnode.deserialize(subcstruct)
 
@@ -779,6 +788,9 @@ class String(SchemaType):
                             mapping={'val':appstruct, 'err':e})
                           )
     def deserialize(self, node, cstruct):
+        if not cstruct:
+            return null
+        
         try:
             result = cstruct
             if not isinstance(result, unicode):
@@ -812,9 +824,9 @@ class Number(SchemaType):
                             mapping={'val':appstruct}),
                           )
     def deserialize(self, node, cstruct):
-        if not cstruct:
-            raise Invalid(node, _('Required'))
-
+        if cstruct != 0 and not cstruct:
+            return null
+        
         try:
             return self.num(cstruct)
         except Exception:
@@ -888,6 +900,9 @@ class Boolean(SchemaType):
         return appstruct and 'true' or 'false'
 
     def deserialize(self, node, cstruct):
+        if cstruct is null:
+            return null
+        
         try:
             result = str(cstruct)
         except:
@@ -1013,6 +1028,9 @@ class GlobalObject(SchemaType):
                             mapping={'val':appstruct})
                           )
     def deserialize(self, node, cstruct):
+        if not cstruct:
+            return null
+        
         if not isinstance(cstruct, basestring):
             raise Invalid(node,
                           _('"${val}" is not a string',
@@ -1096,6 +1114,9 @@ class DateTime(SchemaType):
         return appstruct.isoformat()
 
     def deserialize(self, node, cstruct):
+        if not cstruct:
+            return null
+        
         try:
             result = iso8601.parse_date(cstruct)
         except (iso8601.ParseError, TypeError), e:
@@ -1166,6 +1187,8 @@ class Date(SchemaType):
         return appstruct.isoformat()
 
     def deserialize(self, node, cstruct):
+        if not cstruct:
+            return null
         try:
             result = iso8601.parse_date(cstruct)
             result = result.date()
@@ -1201,11 +1224,11 @@ class SchemaNode(object):
     - ``default``: The default serialization value for this node.
       Default: :attr:`colander.null`.
 
-    - ``missing``: The default deserialization value for this node.
-      If it is not provided, the missing value of this node will be a
-      special marker value, indicating that it is considered
-      'required' (the ``required`` computed attribute will be
-      ``True``).
+    - ``missing``: The default deserialization value for this node.  If it is
+      not provided, the missing value of this node will be the special marker
+      value :attr:`colander.required`, indicating that it is considered
+      'required'.  When ``missing`` is :attr:`colander.required`, the
+      ``required`` computed attribute will be ``True``.
 
     - ``validator``: Optional validator for this node.  It should be
       an object that implements the
@@ -1253,9 +1276,13 @@ class SchemaNode(object):
         self.typ = typ
         self.validator = kw.pop('validator', None)
         self.default = kw.pop('default', null)
-        self.missing = kw.pop('missing', _marker)
+        self.missing = kw.pop('missing', required)
         self.name = kw.pop('name', '')
-        self.title = kw.pop('title', self.name.replace('_', ' ').title())
+        self.raw_title = kw.pop('title', _marker)
+        if self.raw_title is _marker:
+            self.title = self.name.replace('_', ' ').title()
+        else:
+            self.title = self.raw_title
         self.description = kw.pop('description', '')
         self.widget = kw.pop('widget', None)
         self.after_bind = kw.pop('after_bind', None)
@@ -1267,12 +1294,13 @@ class SchemaNode(object):
         """ A property which returns ``True`` if the ``missing`` value
         related to this node was not specified.
 
-        A return value of ``True`` implies that a ``missing`` value
-        wasn't specified for this node.  A return value of ``False``
-        implies that a ``missing`` value was specified for this node."""
+        A return value of ``True`` implies that a ``missing`` value wasn't
+        specified for this node or that the ``missing`` value of this node is
+        :attr:`colander.required`.  A return value of ``False`` implies that
+        a 'real' ``missing`` value was specified for this node."""
         if isinstance(self.missing, deferred):  # unbound schema with deferreds
             return True
-        return self.missing is _marker
+        return self.missing is required
 
     def serialize(self, appstruct=null):
         """ Serialize the :term:`appstruct` to a :term:`cstruct` based
@@ -1308,34 +1336,39 @@ class SchemaNode(object):
         this node using the fstruct passed. """
 
     def deserialize(self, cstruct=null):
-        """ Deserialize and validate the :term:`cstruct` into an
-        :term:`appstruct` based on the schema, and return the
-        deserialized, validated appstruct.  If the cstruct cannot be
-        validated, a :exc:`colander.Invalid` exception will be raised.
+        """ Deserialize the :term:`cstruct` into an :term:`appstruct` based
+        on the schema, and return the deserialized, then validate the
+        resulting appstruct.  The ``cstruct`` value is deserialized into an
+        ``appstruct`` unconditionally.
 
-        If ``cstruct`` is :attr:`colander.null`, do something special:
+        If ``appstruct`` returned by type deserialization is the value
+        :attr:`colander.null`, do something special before attempting
+        validation:
 
-        - If the ``missing`` attribute of this node has been set
-          explicitly, return its value.  No deserialization or
-          validation of this value is performed; it is simply
-          returned.
+        - If the ``missing`` attribute of this node has been set explicitly,
+          return its value.  No validation of this value is performed; it is
+          simply returned.
 
         - If the ``missing`` attribute of this node has not been set
           explicitly, raise a :exc:`colander.Invalid` exception error.
 
+        If the appstruct is not ``colander.null`` and cannot be validated , a
+        :exc:`colander.Invalid` exception will be raised.
+
         If a ``cstruct`` argument is not explicitly provided, it
         defaults to :attr:`colander.null`.
         """
-        if cstruct is null:
+        appstruct = self.typ.deserialize(self, cstruct)
+
+        if appstruct is null:
             appstruct = self.missing
-            if appstruct is _marker:
+            if appstruct is required:
                 raise Invalid(self, _('Required'))
             if isinstance(appstruct, deferred): # unbound schema with deferreds
                 raise Invalid(self, _('Required'))
             # We never deserialize or validate the missing value
             return appstruct
 
-        appstruct = self.typ.deserialize(self, cstruct)
         if self.validator is not None:
             if not isinstance(self.validator, deferred): # unbound
                 self.validator(self, appstruct)
@@ -1393,6 +1426,13 @@ class SchemaNode(object):
         """ Iterate over the children nodes of this schema node """
         return iter(self.children)
 
+    def __contains__(self, name):
+        try:
+            self[name]
+        except KeyError:
+            return False
+        return True
+
     def __repr__(self):
         return '<%s.%s object at %d (named %s)>' % (
             self.__module__,
@@ -1401,14 +1441,13 @@ class SchemaNode(object):
             self.name,
             )
 
-
 class _SchemaMeta(type):
     def __init__(cls, name, bases, clsattrs):
         nodes = []
         for name, value in clsattrs.items():
             if isinstance(value, SchemaNode):
                 value.name = name
-                if not value.title:
+                if value.raw_title is _marker:
                     value.title = name.replace('_', ' ').title()
                 nodes.append((value._order, value))
         cls.__schema_nodes__ = nodes
