@@ -354,6 +354,13 @@ class TestMapping(unittest.TestCase):
         self.failUnless(
             e.msg.interpolate().startswith('"None" is not a mapping type'))
 
+    def test_deserialize_null(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
     def test_deserialize_no_subnodes(self):
         node = DummySchemaNode(None)
         typ = self._makeOne()
@@ -483,6 +490,13 @@ class TestTuple(unittest.TestCase):
         result = typ.deserialize(node, ())
         self.assertEqual(result, ())
 
+    def test_deserialize_null(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
     def test_deserialize_ok(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
@@ -620,6 +634,12 @@ class TestSequence(unittest.TestCase):
         result = typ.deserialize(node, ())
         self.assertEqual(result, [])
 
+    def test_deserialize_no_null(self):
+        import colander
+        typ = self._makeOne()
+        result = typ.deserialize(None, colander.null)
+        self.assertEqual(result, colander.null)
+
     def test_deserialize_ok(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
@@ -711,10 +731,11 @@ class TestString(unittest.TestCase):
         self.assertEqual(Str, String)
 
     def test_deserialize_emptystring(self):
+        from colander import null
         node = DummySchemaNode(None)
         typ = self._makeOne(None)
         result = typ.deserialize(node, '')
-        self.assertEqual(result, '')
+        self.assertEqual(result, null)
 
     def test_deserialize_uncooperative(self):
         val = Uncooperative()
@@ -738,16 +759,16 @@ class TestString(unittest.TestCase):
         self.assertEqual(result, unicode(value))
 
     def test_deserialize_from_utf8(self):
-        utf8 = '\xc3\xb8'
         uni = u'\xf8'
+        utf8 = uni.encode('utf-8')
         node = DummySchemaNode(None)
         typ = self._makeOne('utf-8')
         result = typ.deserialize(node, utf8)
         self.assertEqual(result, uni)
 
     def test_deserialize_from_utf16(self):
-        utf16 = '\xff\xfe\xf8\x00'
         uni = u'\xf8'
+        utf16 = uni.encode('utf-16')
         node = DummySchemaNode(None)
         typ = self._makeOne('utf-16')
         result = typ.deserialize(node, utf16)
@@ -782,16 +803,16 @@ class TestString(unittest.TestCase):
         self.assertEqual(result, value)
 
     def test_serialize_to_utf8(self):
-        utf8 = '\xc3\xb8'
         uni = u'\xf8'
+        utf8 = uni.encode('utf-8')
         node = DummySchemaNode(None)
         typ = self._makeOne('utf-8')
         result = typ.serialize(node, uni)
         self.assertEqual(result, utf8)
 
     def test_serialize_to_utf16(self):
-        utf16 = '\xff\xfe\xf8\x00'
         uni = u'\xf8'
+        utf16 = uni.encode('utf-16')
         node = DummySchemaNode(None)
         typ = self._makeOne('utf-16')
         result = typ.serialize(node, uni)
@@ -822,12 +843,13 @@ class TestInteger(unittest.TestCase):
         result = typ.serialize(node, val)
         self.assertEqual(result, colander.null)
 
-    def test_serialize_emptystring_required(self):
+    def test_serialize_emptystring(self):
+        import colander
         val = ''
         node = DummySchemaNode(None)
         typ = self._makeOne()
-        e = invalid_exc(typ.deserialize, node, val)
-        self.assertEqual(e.msg, 'Required')
+        result = typ.deserialize(node, val)
+        self.assertEqual(result, colander.null)
 
     def test_deserialize_fails(self):
         val = 'P'
@@ -870,12 +892,13 @@ class TestFloat(unittest.TestCase):
         result = typ.serialize(node, val)
         self.assertEqual(result, colander.null)
 
-    def test_serialize_emptystring_required(self):
+    def test_serialize_emptystring(self):
+        import colander
         val = ''
         node = DummySchemaNode(None)
         typ = self._makeOne()
-        e = invalid_exc(typ.deserialize, node, val)
-        self.assertEqual(e.msg, 'Required')
+        result = typ.deserialize(node, val)
+        self.assertEqual(result, colander.null)
 
     def test_deserialize_fails(self):
         val = 'P'
@@ -918,12 +941,13 @@ class TestDecimal(unittest.TestCase):
         result = typ.serialize(node, val)
         self.assertEqual(result, colander.null)
 
-    def test_serialize_emptystring_required(self):
+    def test_serialize_emptystring(self):
+        import colander
         val = ''
         node = DummySchemaNode(None)
         typ = self._makeOne()
-        e = invalid_exc(typ.deserialize, node, val)
-        self.assertEqual(e.msg, 'Required')
+        result = typ.deserialize(node, val)
+        self.assertEqual(result, colander.null)
 
     def test_deserialize_fails(self):
         val = 'P'
@@ -987,6 +1011,13 @@ class TestBoolean(unittest.TestCase):
         e = invalid_exc(typ.deserialize, node, Uncooperative())
         self.failUnless(e.msg.endswith('not a string'))
 
+    def test_deserialize_null(self):
+        import colander
+        typ = self._makeOne()
+        node = DummySchemaNode(None)
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
     def test_serialize(self):
         typ = self._makeOne()
         node = DummySchemaNode(None)
@@ -1110,11 +1141,25 @@ class TestGlobalObject(unittest.TestCase):
         self.assertRaises(ImportError, typ._pkg_resources_style, None,
                           ':notexisting')
 
-    def test_deserialize_not_a_string(self):
+    def test_deserialize_None(self):
+        import colander
         typ = self._makeOne()
         node = DummySchemaNode(None)
-        e = invalid_exc(typ.deserialize, node, None)
-        self.assertEqual(e.msg.interpolate(), '"None" is not a string')
+        result = typ.deserialize(node, None)
+        self.assertEqual(result, colander.null)
+
+    def test_deserialize_null(self):
+        import colander
+        typ = self._makeOne()
+        node = DummySchemaNode(None)
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
+    def test_deserialize_notastring(self):
+        import colander
+        typ = self._makeOne()
+        node = DummySchemaNode(None)
+        self.assertRaises(colander.Invalid, typ.deserialize, node, True)
 
     def test_deserialize_using_pkgresources_style(self):
         typ = self._makeOne()
@@ -1243,6 +1288,20 @@ class TestDateTime(unittest.TestCase):
         e = invalid_exc(typ.deserialize, node, 'garbage')
         self.failUnless('Invalid' in e.msg)
 
+    def test_deserialize_null(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
+    def test_deserialize_empty(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, '')
+        self.assertEqual(result, colander.null)
+
     def test_deserialize_success(self):
         import iso8601
         typ = self._makeOne()
@@ -1309,6 +1368,20 @@ class TestDate(unittest.TestCase):
         e = invalid_exc(typ.deserialize, node, '10-10-10-10')
         self.failUnless('Invalid' in e.msg)
 
+    def test_deserialize_null(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, colander.null)
+        self.assertEqual(result, colander.null)
+
+    def test_deserialize_empty(self):
+        import colander
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        result = typ.deserialize(node, '')
+        self.assertEqual(result, colander.null)
+
     def test_deserialize_success_date(self):
         typ = self._makeOne()
         date = self._today()
@@ -1503,6 +1576,13 @@ class TestSchemaNode(unittest.TestCase):
         it = node.__iter__()
         self.assertEqual(list(it), ['a', 'b', 'c'])
 
+    def test___contains__(self):
+        node = self._makeOne(None)
+        another = self._makeOne(None, name='another')
+        node.add(another)
+        self.assertEquals('another' in node, True)
+        self.assertEquals('b' in node, False)
+
     def test_clone(self):
         inner_typ = DummyType()
         outer_typ = DummyType()
@@ -1608,6 +1688,19 @@ class TestSchema(unittest.TestCase):
         self.assertEqual(node.children[0].title, 'Thing A')
         self.assertEqual(node.children[1].title, 'bar')
 
+    def test_title_munging(self):
+        import colander
+        class MySchema(colander.Schema):
+            thing1 = colander.SchemaNode(colander.String())
+            thing2 = colander.SchemaNode(colander.String(), title=None)
+            thing3 = colander.SchemaNode(colander.String(), title='')
+            thing4 = colander.SchemaNode(colander.String(), title='thing2')
+        node = MySchema()
+        self.assertEqual(node.children[0].title, 'Thing1')
+        self.assertEqual(node.children[1].title, None)
+        self.assertEqual(node.children[2].title, '')
+        self.assertEqual(node.children[3].title, 'thing2')
+
 class TestSequenceSchema(unittest.TestCase):
     def test_succeed(self):
         import colander
